var View, htmlEscape, htmlParser, parse, quoteAttr, simpleView;
htmlParser = require('./htmlParser');
View = module.exports = function() {
  var self;
  self = this;
  this._views = {};
  this._loadFuncs = '';
  this.uniqueId = function() {
    return '_' + (self._idCount++).toString(36);
  };
};
View.prototype = {
  get: function(view, obj) {
    var item;
    if (view = this._views[view]) {
      if (Array.isArray(obj)) {
        return ((function() {
          var _i, _len, _results;
          _results = [];
          for (_i = 0, _len = obj.length; _i < _len; _i++) {
            item = obj[_i];
            _results.push(view(item));
          }
          return _results;
        })()).join('');
      } else {
        return view(obj);
      }
    } else {
      return '';
    }
  },
  preLoad: function(fn) {
    return this._loadFuncs += "(" + fn + ")();";
  },
  make: function(name, data, template, options) {
    var after, render, self, uniqueId;
    after = options && options.after;
    uniqueId = this.uniqueId;
    self = this;
    render = function() {
      render = template ? parse(template, uniqueId, self) : simpleView(name, self);
      return render.apply(null, arguments);
    };
    return this._register(name, after, (typeof data === 'function' ? function() {
      return render(data.apply(null, arguments));
    } : function() {
      return render(data);
    }));
  },
  _register: function(name, after, fn) {
    return this._views[name] = after ? function() {
      setTimeout(after, 0);
      return fn.apply(null, arguments);
    } : fn;
  }
};
View.htmlEscape = htmlEscape = function(s) {
  if (s == null) {
    return '';
  } else {
    return s.toString().replace(/[&<>]/g, function(s) {
      switch (s) {
        case '&':
          return '&amp;';
        case '<':
          return '&lt;';
        case '>':
          return '&gt;';
        default:
          return s;
      }
    });
  }
};
quoteAttr = function(s) {
  if (s == null || s === '') {
    return '""';
  }
  s = s.toString().replace(/"/g, '&quot;');
  if (/[ =]/.test(s)) {
    return '"' + s + '"';
  } else {
    return s;
  }
};
parse = function(template, uniqueId, view) {
  var elementParse, events, extractPlaceholder, html, htmlIndex, item, key, modelText, pushValue, stack, value, _i, _len, _ref;
  modelText = function(name, escaped, quote) {
    return function(data) {
      var datum, obj, text;
      datum = data[name];
      obj = datum.model ? view.model.get(datum.model) : datum;
      text = datum.view ? view.get(datum.view, obj) : obj;
      if (escaped) {
        text = htmlEscape(text);
      }
      if (quote) {
        text = quoteAttr(text);
      }
      return text;
    };
  };
  extractPlaceholder = function(text) {
    var match;
    match = /^(.*?)(\{{2,3})(\w+)\}{2,3}(.*)$/.exec(text);
    if (match) {
      return {
        pre: match[1],
        escaped: match[2] === '{{',
        name: match[3],
        post: match[4]
      };
    } else {
      return null;
    }
  };
  stack = [];
  events = [];
  html = [''];
  htmlIndex = 0;
  elementParse = {
    input: function(attr, attrs, name) {
      var method, silent;
      if (attr !== 'value') {
        return 'attr';
      }
      if ('silent' in attrs) {
        delete attrs.silent;
        method = 'prop';
        silent = 1;
      } else {
        method = 'propPolite';
        silent = 0;
      }
      events.push(function(data) {
        var domArgs, domEvents;
        domArgs = ['set', data[name].model, attrs._id || attrs.id, 'prop', 'value', silent];
        domEvents = view.dom.events;
        domEvents.bind('keyup', domArgs);
        return domEvents.bind('keydown', domArgs);
      });
      return method;
    }
  };
  htmlParser.parse(template, {
    start: function(tag, attrs) {
      var key, match, method, name, value;
      for (key in attrs) {
        value = attrs[key];
        if (match = extractPlaceholder(value)) {
          name = match.name;
          if (attrs.id === void 0) {
            attrs.id = function() {
              return attrs._id = uniqueId();
            };
          }
          method = tag in elementParse ? elementParse[tag](key, attrs, name) : 'attr';
          events.push(function(data) {
            var path;
            path = data[name].model;
            if (path) {
              return view.model.__events.bind(path, [attrs._id || attrs.id, method, key]);
            }
          });
          attrs[key] = modelText(name, match.escaped, true);
        }
      }
      return stack.push(['start', tag, attrs]);
    },
    chars: function(text) {
      var attrs, escaped, last, match, name, post, pre;
      if (match = extractPlaceholder(text)) {
        name = match.name;
        escaped = +match.escaped;
        pre = match.pre;
        post = match.post;
        if (pre) {
          stack.push(['chars', pre]);
        }
        if (pre || post) {
          stack.push(['start', 'span', {}]);
        }
        text = modelText(name, escaped);
        last = stack[stack.length - 1];
        if (last[0] === 'start') {
          attrs = last[2];
          if (attrs.id === void 0) {
            attrs.id = function() {
              return attrs._id = uniqueId();
            };
          }
          events.push(function(data) {
            var params, path, viewFunc;
            if (path = data[name].model) {
              params = [attrs._id || attrs.id, 'html', escaped];
              if (viewFunc = data[name].view) {
                params[3] = viewFunc;
              }
              return view.model.__events.bind(path, params);
            }
          });
        }
      }
      if (text) {
        stack.push(['chars', text]);
      }
      if (pre || post) {
        stack.push(['end', 'span']);
      }
      if (post) {
        return htmlParse.chars(post);
      }
    },
    end: function(tag) {
      return stack.push(['end', tag]);
    }
  });
  for (_i = 0, _len = stack.length; _i < _len; _i++) {
    item = stack[_i];
    pushValue = function(value, quote) {
      if (value && value.call) {
        return htmlIndex = html.push(value, '') - 1;
      } else {
        return html[htmlIndex] += quote ? quoteAttr(value) : value;
      }
    };
    switch (item[0]) {
      case 'start':
        html[htmlIndex] += '<' + item[1];
        _ref = item[2];
        for (key in _ref) {
          value = _ref[key];
          html[htmlIndex] += ' ' + key + '=';
          pushValue(value, true);
        }
        html[htmlIndex] += '>';
        break;
      case 'chars':
        pushValue(item[1]);
        break;
      case 'end':
        html[htmlIndex] += '</' + item[1] + '>';
    }
  }
  return function(data, obj) {
    var event, item, rendered, _j, _len2;
    rendered = ((function() {
      var _j, _len2, _results;
      _results = [];
      for (_j = 0, _len2 = html.length; _j < _len2; _j++) {
        item = html[_j];
        _results.push((item && item.call ? item(data) : item));
      }
      return _results;
    })()).join('');
    for (_j = 0, _len2 = events.length; _j < _len2; _j++) {
      event = events[_j];
      event(data);
    }
    return rendered;
  };
};
simpleView = function(name, view) {
  return function(datum) {
    var model, obj, path, text;
    path = datum.model;
    model = view.model;
    obj = path ? model.get(path) : datum;
    text = datum.view ? view.get(datum.view, obj) : obj;
    if (path && name === 'Title') {
      model.__events.bind(path, ['$doc', 'prop', 'title']);
    }
    return text && text.replace(/\n/g, '');
  };
};